<?php

/*********************************************************************

 スケジュールdisplayプラグイン (2014/01/30)

 freo.jp
 customize by 32877(http://32877.xii.jp)

*********************************************************************/

/* メイン処理 */
function freo_display_schedule()
{
	global $freo;

	//パラメータ検証
	if (!isset($_GET['page']) or !preg_match('/^\d+$/', $_GET['page']) or $_GET['page'] < 1) {
		$_GET['page'] = 1;
	}

	//検索条件設定
	$condition = null;
	if (isset($_GET['word'])) {
		$condition .= ' AND title LIKE ' . $freo->pdo->quote('%' . $_GET['word'] . '%') . ' OR text LIKE ' . $freo->pdo->quote('%' . $_GET['word'] . '%') . ' OR place LIKE ' . $freo->pdo->quote('%' . $_GET['word'] . '%');
	}
	if (isset($_GET['category_id'])) {
		$condition .= ' AND category_id = ' . $freo->pdo->quote($_GET['category_id']);
	}
	if (isset($_GET['tag'])) {
		$condition .= ' AND tag = ' . $freo->pdo->quote($_GET['tag']) . ' OR tag LIKE ' . $freo->pdo->quote($_GET['tag'] . ',%') . ' OR tag LIKE ' . $freo->pdo->quote('%,' . $_GET['tag']) . ' OR tag LIKE ' . $freo->pdo->quote('%,' . $_GET['tag'] . ',%');
	}
	if (isset($_GET['date'])) {
		if (preg_match('/^\d\d\d\d$/', $_GET['date'])) {
			if (FREO_DATABASE_TYPE == 'mysql') {
				$condition .= ' AND DATE_FORMAT(datetime, \'%Y\') = ' . $freo->pdo->quote($_GET['date']);
			} else {
				$condition .= ' AND STRFTIME(\'%Y\', datetime) = ' . $freo->pdo->quote($_GET['date']);
			}
		} elseif (preg_match('/^\d\d\d\d\d\d$/', $_GET['date'])) {
			if (FREO_DATABASE_TYPE == 'mysql') {
				$condition .= ' AND DATE_FORMAT(datetime, \'%Y%m\') = ' . $freo->pdo->quote($_GET['date']);
			} else {
				$condition .= ' AND STRFTIME(\'%Y%m\', datetime) = ' . $freo->pdo->quote($_GET['date']);
			}
		} elseif (preg_match('/^\d\d\d\d\d\d\d\d$/', $_GET['date'])) {
			if (FREO_DATABASE_TYPE == 'mysql') {
				$condition .= ' AND DATE_FORMAT(datetime, \'%Y%m%d\') = ' . $freo->pdo->quote($_GET['date']);
			} else {
				$condition .= ' AND STRFTIME(\'%Y%m%d\', datetime) = ' . $freo->pdo->quote($_GET['date']);
			}
		}
	}


	if ($condition) {
		$condition = ' WHERE id IS NOT NULL ' . $condition;
	}

	//検索条件
	$ascending = false;

		if ($freo->config['plugin']['schedule']['schedule_order'] == 'ascend') {
			$ascending = true;
		}

	if ($ascending == true) {
		$order = ' datetime';
	} else {
		$order = ' datetime DESC';
	}
	
	$titleascending = false;

		if ($freo->config['plugin']['schedule']['schedule_order2'] == 'ascend') {
			$titleascending = true;
		}

	if ($titleascending == true) {
		$titleorder = ' title';
	} else {
		$titleorder = ' title DESC';
	}
	
	//予定取得
	$stmt = $freo->pdo->prepare('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_schedules ' . $condition . ' ORDER BY' . $order . ' , ' . $titleorder . ' LIMIT :start, :limit');
	$stmt->bindValue(':start', intval($freo->config['plugin']['schedule']['utility_limit']) * ($_GET['page'] - 1), PDO::PARAM_INT);
	$stmt->bindValue(':limit', intval($freo->config['plugin']['schedule']['utility_limit']), PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		freo_error($stmt->errorInfo());
	}

	$plugin_schedules = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$plugin_schedules[$data['id']] = $data;
	}

	//予定ID取得
	$plugin_schedule_keys = array_keys($plugin_schedules);

	//予定タグ取得
	$plugin_schedule_tags = array();
	foreach ($plugin_schedule_keys as $plugin_schedule) {
		if (!$plugin_schedules[$plugin_schedule]['tag']) {
			continue;
		}

		$plugin_schedule_tags[$plugin_schedule] = explode(',', $plugin_schedules[$plugin_schedule]['tag']);
	}

	//ファイル取得
	$plugin_schedule_files = array();
	foreach ($plugin_schedule_keys as $plugin_schedule) {
		$file_dir = FREO_FILE_DIR . 'plugins/schedule_files/' . $plugin_schedule . '/';

		if (file_exists($file_dir)) {
			if ($dir = scandir($file_dir)) {
				foreach ($dir as $data) {
					if (is_file($file_dir . $data) and preg_match("/(\w+)\.\w+$/", $data, $matches)) {
						$plugin_schedule_files[$plugin_schedule][$matches[1]] = $data;
					}
				}
			} else {
				freo_error('商品ファイル保存ディレクトリを開けません。');
			}
		}
	}

	//予定数・ページ数取得
	$stmt = $freo->pdo->query('SELECT COUNT(*) FROM ' . FREO_DATABASE_PREFIX . 'plugin_schedules ' . $condition);
	if (!$stmt) {
		freo_error($freo->pdo->errorInfo());
	}

	$data                  = $stmt->fetch(PDO::FETCH_NUM);
	$plugin_schedule_count = $data[0];
	$plugin_schedule_page  = ceil($plugin_schedule_count / $freo->config['plugin']['schedule']['default_limit']);

	//カテゴリー取得
	$stmt = $freo->pdo->query('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_schedule_categories ORDER BY sort, id');
	if (!$stmt) {
		freo_error($freo->pdo->errorInfo());
	}

	$plugin_schedule_categories = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$plugin_schedule_categories[$data['id']] = $data;
	}

	//データ割当
	$freo->smarty->assign(array(
		'token'                 => freo_token('create'),
		'plugin_schedules'      => $plugin_schedules,
		'plugin_schedule_tags'  => $plugin_schedule_tags,
		'plugin_schedule_files' => $plugin_schedule_files,
		'plugin_schedule_count' => $plugin_schedule_count,
		'plugin_schedule_page'  => $plugin_schedule_page,
		'plugin_schedule_categories'  => $plugin_schedule_categories
	));
	return;
}

?>
