<?php

/*********************************************************************

 スケジュールプラグイン (2016/01/14)

 freo.jp
 customize by 32877(http://32877.xii.jp)

*********************************************************************/

/* メイン処理 */
function freo_page_schedule()
{
	global $freo;

	switch ($_REQUEST['freo']['work']) {
		case 'setup':
			freo_page_schedule_setup();
			break;
		case 'setup_execute':
			freo_page_schedule_setup_execute();
			break;
		case 'admin':
			freo_page_schedule_admin();
			break;
		case 'admin_form':
			freo_page_schedule_admin_form();
			break;
		case 'admin_post':
			freo_page_schedule_admin_post();
			break;
		case 'admin_delete':
			freo_page_schedule_admin_delete();
			break;
		case 'admin_category':
			freo_page_schedule_admin_category();
			break;
		case 'admin_category_form':
			freo_page_schedule_admin_category_form();
			break;
		case 'admin_category_post':
			freo_page_schedule_admin_category_post();
			break;
		case 'admin_category_update':
			freo_page_schedule_admin_category_update();
			break;
		case 'admin_category_delete':
			freo_page_schedule_admin_category_delete();
			break;
		case 'view':
			freo_page_schedule_view();
			break;
		default:
			freo_page_schedule_default();
	}

	return;
}

/* セットアップ */
function freo_page_schedule_setup()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//リクエストメソッドに応じた処理を実行
	if ($_SERVER['REQUEST_METHOD'] == 'POST') {
		//ワンタイムトークン確認
		if (!freo_token('check')) {
			$freo->smarty->append('errors', '不正なアクセスです。');
		}

		//エラー確認
		if (!$freo->smarty->get_template_vars('errors')) {
			freo_redirect('schedule/setup_execute?freo%5Btoken%5D=' . freo_token('create'), true);
		}
	}

	//データ割当
	$freo->smarty->assign(array(
		'token'       => freo_token('create'),
		'plugin_id'   => 'schedule',
		'plugin_name' => FREO_PLUGIN_SCHEDULE_NAME
	));

	//データ出力
	freo_output('plugins/setup.html');

	return;
}

/* セットアップ | セットアップ実行 */
function freo_page_schedule_setup_execute()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//ワンタイムトークン確認
	if (!freo_token('check')) {
		freo_redirect('setup?error=1', true);
	}

	//データベーステーブル存在検証
	if (FREO_DATABASE_TYPE == 'mysql') {
		$query = 'SHOW TABLES';
	} else {
		$query = 'SELECT name FROM sqlite_master WHERE type = \'table\'';
	}
	$stmt = $freo->pdo->query($query);
	if (!$stmt) {
		freo_error($freo->pdo->errorInfo());
	}

	$table = array();
	while ($data = $stmt->fetch(PDO::FETCH_NUM)) {
		$table[$data[0]] = true;
	}

	//データベーステーブル定義
	if (FREO_DATABASE_TYPE == 'mysql') {
		$queries = array(
			'plugin_schedules' => '(
				id INT UNSIGNED NOT NULL AUTO_INCREMENT,
				category_id VARCHAR(255),
				created DATETIME NOT NULL,
				modified DATETIME NOT NULL,
				datetime DATETIME NOT NULL,
				enddatetime DATETIME,
				title VARCHAR(255),
				url TEXT NOT NULL,
				tag VARCHAR(255),
				place VARCHAR(255),
				option01    TEXT,
				option02    TEXT,
				option03    TEXT,
				option04    TEXT,
				option05    TEXT,
				option06    TEXT,
				option07    TEXT,
				option08    TEXT,
				option09    TEXT,
				option10    TEXT,
				text TEXT,
				PRIMARY KEY(id)
			)',
			'plugin_schedule_categories' => '(
				id       VARCHAR(80)  NOT NULL,
				created  DATETIME     NOT NULL,
				modified DATETIME     NOT NULL,
				sort     INT UNSIGNED NOT NULL,
				name     VARCHAR(255) NOT NULL,
				memo     TEXT,
				PRIMARY KEY(id)
			)'
		);
	} else {
		$queries = array(
			'plugin_schedules' => '(
				id INTEGER,
				category_id VARCHAR,
				created DATETIME NOT NULL,
				modified DATETIME NOT NULL,
				datetime DATETIME NOT NULL,
				enddatetime DATETIME,
				title VARCHAR,
				url TEXT,
				tag VARCHAR,
				place VARCHAR,
				option01    TEXT,
				option02    TEXT,
				option03    TEXT,
				option04    TEXT,
				option05    TEXT,
				option06    TEXT,
				option07    TEXT,
				option08    TEXT,
				option09    TEXT,
				option10    TEXT,
				text TEXT,
				PRIMARY KEY(id)
			)',
			'plugin_schedule_categories' => '(
				id       VARCHAR          NOT NULL,
				created  DATETIME         NOT NULL,
				modified DATETIME         NOT NULL,
				sort     INTEGER UNSIGNED NOT NULL,
				name     VARCHAR          NOT NULL,
				memo     TEXT,
				PRIMARY KEY(id)
			)'
		);
	}

	//データベーステーブル作成
	foreach ($queries as $name => $query) {
		if (empty($table[FREO_DATABASE_PREFIX . $name])) {
			$stmt = $freo->pdo->query('CREATE TABLE ' . FREO_DATABASE_PREFIX . $name . $query);
			if (!$stmt) {
				freo_error($freo->pdo->errorInfo());
			}
		}
	}

	//ログ記録
	freo_log(FREO_PLUGIN_SCHEDULE_NAME . 'のセットアップを実行しました。');

	//完了画面へ移動
	freo_redirect('schedule/setup?exec=setup', true);

	return;
}

/* 管理画面 | 予定管理 */
function freo_page_schedule_admin()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//パラメータ検証
	if (!isset($_GET['page']) or !preg_match('/^\d+$/', $_GET['page']) or $_GET['page'] < 1) {
		$_GET['page'] = 1;
	}

	//検索条件設定
	$condition = null;
	if (isset($_GET['word'])) {
		$condition .= ' AND title LIKE ' . $freo->pdo->quote('%' . $_GET['word'] . '%') . ' OR text LIKE ' . $freo->pdo->quote('%' . $_GET['word'] . '%') . ' OR place LIKE ' . $freo->pdo->quote('%' . $_GET['word'] . '%');
	}
	if (isset($_GET['category_id'])) {
		$condition .= ' AND category_id = ' . $freo->pdo->quote($_GET['category_id']);
	}
	if (isset($_GET['tag'])) {
		$condition .= ' AND tag = ' . $freo->pdo->quote($_GET['tag']) . ' OR tag LIKE ' . $freo->pdo->quote($_GET['tag'] . ',%') . ' OR tag LIKE ' . $freo->pdo->quote('%,' . $_GET['tag']) . ' OR tag LIKE ' . $freo->pdo->quote('%,' . $_GET['tag'] . ',%');
	}
	if (isset($_GET['date'])) {
		if (preg_match('/^\d\d\d\d$/', $_GET['date'])) {
			if (FREO_DATABASE_TYPE == 'mysql') {
				$condition .= ' AND DATE_FORMAT(datetime, \'%Y\') = ' . $freo->pdo->quote($_GET['date']);
			} else {
				$condition .= ' AND STRFTIME(\'%Y\', datetime) = ' . $freo->pdo->quote($_GET['date']);
			}
		} elseif (preg_match('/^\d\d\d\d\d\d$/', $_GET['date'])) {
			if (FREO_DATABASE_TYPE == 'mysql') {
				$condition .= ' AND DATE_FORMAT(datetime, \'%Y%m\') = ' . $freo->pdo->quote($_GET['date']);
			} else {
				$condition .= ' AND STRFTIME(\'%Y%m\', datetime) = ' . $freo->pdo->quote($_GET['date']);
			}
		} elseif (preg_match('/^\d\d\d\d\d\d\d\d$/', $_GET['date'])) {
			if (FREO_DATABASE_TYPE == 'mysql') {
				$condition .= ' AND DATE_FORMAT(datetime, \'%Y%m%d\') = ' . $freo->pdo->quote($_GET['date']);
			} else {
				$condition .= ' AND STRFTIME(\'%Y%m%d\', datetime) = ' . $freo->pdo->quote($_GET['date']);
			}
		}
	}
	if ($condition) {
		$condition = ' WHERE id IS NOT NULL ' . $condition;
	}

	//予定取得
	$stmt = $freo->pdo->prepare('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_schedules ' . $condition . ' ORDER BY id DESC LIMIT :start, :limit');
	$stmt->bindValue(':start', intval($freo->config['plugin']['schedule']['admin_limit']) * ($_GET['page'] - 1), PDO::PARAM_INT);
	$stmt->bindValue(':limit', intval($freo->config['plugin']['schedule']['admin_limit']), PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		freo_error($stmt->errorInfo());
	}

	$plugin_schedules = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$plugin_schedules[$data['id']] = $data;
	}

	//予定数・ページ数取得
	$stmt = $freo->pdo->query('SELECT COUNT(*) FROM ' . FREO_DATABASE_PREFIX . 'plugin_schedules ' . $condition);
	if (!$stmt) {
		freo_error($freo->pdo->errorInfo());
	}

	$data                  = $stmt->fetch(PDO::FETCH_NUM);
	$plugin_schedule_count = $data[0];
	$plugin_schedule_page  = ceil($plugin_schedule_count / $freo->config['plugin']['schedule']['admin_limit']);

	//カテゴリー取得
	$stmt = $freo->pdo->query('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_schedule_categories ORDER BY sort, id');
	if (!$stmt) {
		freo_error($freo->pdo->errorInfo());
	}

	$plugin_schedule_categories = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$plugin_schedule_categories[$data['id']] = $data;
	}

	//データ割当
	$freo->smarty->assign(array(
		'token'                 => freo_token('create'),
		'plugin_schedules'      => $plugin_schedules,
		'plugin_schedule_count' => $plugin_schedule_count,
		'plugin_schedule_page'  => $plugin_schedule_page,
		'plugin_schedule_categories'  => $plugin_schedule_categories
	));

	return;
}

/* 管理画面 | 予定入力 */
function freo_page_schedule_admin_form()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//ファイル番号定義
	$numbers = array('', '01', '02', '03', '04', '05', '06', '07', '08', '09', '10');

	//パラメータ検証
	if (!isset($_GET['id']) or !preg_match('/^\d+$/', $_GET['id']) or $_GET['id'] < 1) {
		$_GET['id'] = 0;
	}

	//リクエストメソッドに応じた処理を実行
	if ($_SERVER['REQUEST_METHOD'] == 'POST') {
		//ワンタイムトークン確認
		if (!freo_token('check')) {
			$freo->smarty->append('errors', '不正なアクセスです。');
		}

		//日時データ取得
		if (is_array($_POST['plugin_schedule']['datetime'])) {
			$year   = mb_convert_kana($_POST['plugin_schedule']['datetime']['year'], 'n', 'UTF-8');
			$month  = mb_convert_kana($_POST['plugin_schedule']['datetime']['month'], 'n', 'UTF-8');
			$day    = mb_convert_kana($_POST['plugin_schedule']['datetime']['day'], 'n', 'UTF-8');
			$hour   = mb_convert_kana($_POST['plugin_schedule']['datetime']['hour'], 'n', 'UTF-8');
			$minute = mb_convert_kana($_POST['plugin_schedule']['datetime']['minute'], 'n', 'UTF-8');
			$second = mb_convert_kana($_POST['plugin_schedule']['datetime']['second'], 'n', 'UTF-8');

			$_POST['plugin_schedule']['datetime'] = sprintf('%04d-%02d-%02d %02d:%02d:%02d', $year, $month, $day, $hour, $minute, $second);
		}

		//終了日時データ取得
		if (is_array($_POST['plugin_schedule']['enddatetime'])) {
			$year   = mb_convert_kana($_POST['plugin_schedule']['enddatetime']['year'], 'n', 'UTF-8');
			$month  = mb_convert_kana($_POST['plugin_schedule']['enddatetime']['month'], 'n', 'UTF-8');
			$day    = mb_convert_kana($_POST['plugin_schedule']['enddatetime']['day'], 'n', 'UTF-8');
			$hour   = mb_convert_kana($_POST['plugin_schedule']['enddatetime']['hour'], 'n', 'UTF-8');
			$minute = mb_convert_kana($_POST['plugin_schedule']['enddatetime']['minute'], 'n', 'UTF-8');
			$second = mb_convert_kana($_POST['plugin_schedule']['enddatetime']['second'], 'n', 'UTF-8');

			$_POST['plugin_schedule']['enddatetime'] = sprintf('%04d-%02d-%02d %02d:%02d:%02d', $year, $month, $day, $hour, $minute, $second);
		}

		//アップロードデータ初期化
		foreach ($numbers as $number) {
			if (!isset($_FILES['plugin_schedule']['tmp_name']['file' . $number])) {
				$_FILES['plugin_schedule']['tmp_name']['file' . $number] = null;
			}
		}

		//アップロードデータ取得
		foreach ($numbers as $number) {
			if (is_uploaded_file($_FILES['plugin_schedule']['tmp_name']['file' . $number])) {
				$_POST['plugin_schedule']['file' . $number] = $_FILES['plugin_schedule']['name']['file' . $number];
			} elseif (!isset($_POST['page']['file' . $number])) {
				$_POST['plugin_schedule']['file' . $number] = null;
			}
		}

		//入力データ検証
		if (!$freo->smarty->get_template_vars('errors')) {

			//日時
			if ($_POST['plugin_schedule']['datetime'] == '') {
				$freo->smarty->append('errors', '日時が入力されていません。');
			}

			//タイトル
			if (mb_strlen($_POST['plugin_schedule']['title'], 'UTF-8') > 200) {
				$freo->smarty->append('errors', 'タイトルは200文字以内で入力してください。');
			}

			//URL
			if (mb_strlen($_POST['plugin_schedule']['url'], 'UTF-8') > 5000) {
				$freo->smarty->append('errors', 'URLは5000文字以内で入力してください。');
			}

			//場所
			if (mb_strlen($_POST['plugin_schedule']['place'], 'UTF-8') > 200) {
				$freo->smarty->append('errors', '場所は200文字以内で入力してください。');
			}

			//本文
			if (mb_strlen($_POST['plugin_schedule']['text'], 'UTF-8') > 5000) {
				$freo->smarty->append('errors', '場所は5000文字以内で入力してください。');
			}

			//タグ
			if (mb_strlen($_POST['plugin_schedule']['tag'], 'UTF-8') > 80) {
				$freo->smarty->append('errors', 'タグは80文字以内で入力してください。');
			}
			//イメージ
			if ($_POST['plugin_schedule']['file'] != '') {
				if (!preg_match('/\.(gif|jpeg|jpg|jpe|png)$/i', $_POST['plugin_schedule']['file'])) {
					$freo->smarty->append('errors', 'アップロードできるイメージはGIF、JPEG、PNGのみです。');
				}
			}

		}
		
		//ファイルアップロード
		foreach ($numbers as $number) {
			$file_flag  = false;

			if (!$freo->smarty->get_template_vars('errors')) {
				if (is_uploaded_file($_FILES['plugin_schedule']['tmp_name']['file' . $number])) {
					$temporary_dir  = FREO_FILE_DIR . 'temporaries/plugins/schedule_files/';
					$temporary_file = $_FILES['plugin_schedule']['name']['file' . $number];

					if (move_uploaded_file($_FILES['plugin_schedule']['tmp_name']['file' . $number], $temporary_dir . $temporary_file)) {
						if (preg_match('/\.(.*)$/', $temporary_file, $matches)) {
							$_POST['plugin_schedule']['file' . $number] = 'file' . $number . '.' . $matches[1];

							if (rename($temporary_dir . $temporary_file, $temporary_dir . $_POST['plugin_schedule']['file' . $number])) {
								$temporary_file = $_POST['plugin_schedule']['file' . $number];
							} else {
								$freo->smarty->append('errors', 'ファイル ' . $temporary_dir . $temporary_file . ' の名前を変更できません。');
							}
						} else {
							$freo->smarty->append('errors', 'ファイル ' . $temporary_file . ' の拡張子を取得できません。');
						}

						chmod($temporary_dir . $temporary_file, FREO_PERMISSION_FILE);

						$file_flag = true;
					} else {
						$freo->smarty->append('errors', '予定ファイルをアップロードできません。');
					}
				}
			}

			if (is_uploaded_file($_FILES['plugin_schedule']['tmp_name']['file' . $number]) and !$file_flag) {
				$_POST['plugin_schedule']['file' . $number] = null;
			}
		}

		//エラー確認
		if ($freo->smarty->get_template_vars('errors')) {
			//エラー表示
			$plugin_schedule = $_POST['plugin_schedule'];
		} else {
			$_SESSION['input'] = $_POST;

			//登録処理へ移動
			freo_redirect('schedule/admin_post?freo%5Btoken%5D=' . freo_token('create') . ($_GET['id'] ? '&id=' . $_GET['id'] : ''));
		}
	} else {
		if ($_GET['id']) {
			//編集データ取得
			$stmt = $freo->pdo->prepare('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_schedules WHERE id = :id');
			$stmt->bindValue(':id', $_GET['id'], PDO::PARAM_INT);
			$flag = $stmt->execute();
			if (!$flag) {
				freo_error($stmt->errorInfo());
			}

			if ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
				$plugin_schedule = $data;
			} else {
				freo_error('指定された予定が見つかりません。');
			}
			
			//ファイル取得
			$file_dir = FREO_FILE_DIR . 'plugins/schedule_files/' . $_GET['id'] . '/';

			if (file_exists($file_dir)) {
				if ($dir = scandir($file_dir)) {
					foreach ($dir as $data) {
						if (is_file($file_dir . $data) and preg_match("/(\w+)\.\w+$/", $data, $matches)) {
							$plugin_schedule[$matches[1]] = $data;
						}
					}
				} else {
					freo_error('ファイル保存ディレクトリを開けません。');
				}
			}
		} else {
			//新規データ設定
			$plugin_schedule = array(
				'datetime' => date('Y-m-d'),
				'enddatetime' => date('Y-m-d'),
			);
		}
	}

	//カテゴリー取得
	$stmt = $freo->pdo->query('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_schedule_categories ORDER BY sort, id');
	if (!$stmt) {
		freo_error($freo->pdo->errorInfo());
	}

	$plugin_schedule_categories = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$plugin_schedule_categories[$data['id']] = $data;
	}

	//データ割当
	$freo->smarty->assign(array(
		'token' => freo_token('create'),
		'plugin_schedule_categories' => $plugin_schedule_categories,
		'input' => array(
			'plugin_schedule' => $plugin_schedule
		)
	));

	return;
}

/* 管理画面 | スケジュール */
function freo_page_schedule_admin_post()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//入力データ確認
	if (empty($_SESSION['input'])) {
		freo_redirect('schedule/admin?error=1');
	}

	//ワンタイムトークン確認
	if (!freo_token('check')) {
		freo_redirect('schedule/admin?error=1');
	}

	//ファイル番号定義
	$numbers = array('', '01', '02', '03', '04', '05', '06', '07', '08', '09', '10');

	//入力データ取得
	$schedule = $_SESSION['input']['plugin_schedule'];

	if ($schedule['category_id'] == '') {
		$schedule['category_id'] = null;
	}
	if ($schedule['tag'] == '') {
		$schedule['tag'] = null;
	}
	if ($schedule['text'] == '') {
		$schedule['text'] = null;
	}
	if ($schedule['option01'] == '') {
		$schedule['option01'] = null;
	}
	if ($schedule['option02'] == '') {
		$schedule['option02'] = null;
	}
	if ($schedule['option03'] == '') {
		$schedule['option03'] = null;
	}
	if ($schedule['option04'] == '') {
		$schedule['option04'] = null;
	}
	if ($schedule['option05'] == '') {
		$schedule['option05'] = null;
	}
	if ($schedule['option06'] == '') {
		$schedule['option06'] = null;
	}
	if ($schedule['option07'] == '') {
		$schedule['option07'] = null;
	}
	if ($schedule['option08'] == '') {
		$schedule['option08'] = null;
	}
	if ($schedule['option09'] == '') {
		$schedule['option09'] = null;
	}
	if ($schedule['option10'] == '') {
		$schedule['option10'] = null;
	}

	//データ登録
	if (isset($_GET['id'])) {
		$stmt = $freo->pdo->prepare('UPDATE ' . FREO_DATABASE_PREFIX . 'plugin_schedules SET category_id = :category_id, modified = :now, datetime = :datetime, enddatetime = :enddatetime, title = :title, url = :url, tag = :tag, place = :place, text = :text, option01 = :option01, option02 = :option02, option03 = :option03, option04 = :option04, option05 = :option05, option06 = :option06, option07 = :option07, option08 = :option08, option09 = :option09, option10 = :option10 WHERE id = :id');
		$stmt->bindValue(':category_id', $schedule['category_id']);
		$stmt->bindValue(':now',         date('Y-m-d H:i:s'));
		$stmt->bindValue(':datetime',    $schedule['datetime']);
		$stmt->bindValue(':enddatetime', $schedule['enddatetime']);
		$stmt->bindValue(':title',       $schedule['title']);
		$stmt->bindValue(':url',         $schedule['url']);
		$stmt->bindValue(':tag',         $schedule['tag']);
		$stmt->bindValue(':place',       $schedule['place']);
		$stmt->bindValue(':text',        $schedule['text']);
		$stmt->bindValue(':id',          $schedule['id'], PDO::PARAM_INT);
		$stmt->bindValue(':option01',    $schedule['option01']);
		$stmt->bindValue(':option02',    $schedule['option02']);
		$stmt->bindValue(':option03',    $schedule['option03']);
		$stmt->bindValue(':option04',    $schedule['option04']);
		$stmt->bindValue(':option05',    $schedule['option05']);
		$stmt->bindValue(':option06',    $schedule['option06']);
		$stmt->bindValue(':option07',    $schedule['option07']);
		$stmt->bindValue(':option08',    $schedule['option08']);
		$stmt->bindValue(':option09',    $schedule['option09']);
		$stmt->bindValue(':option10',    $schedule['option10']);
		$flag = $stmt->execute();
		if (!$flag) {
			freo_error($stmt->errorInfo());
		}
	} else {
		$stmt = $freo->pdo->prepare('INSERT INTO ' . FREO_DATABASE_PREFIX . 'plugin_schedules VALUES(NULL, :category_id, :now1, :now2,  :datetime, :enddatetime, :title, :url, :tag, :place, :option01, :option02, :option03, :option04, :option05, :option06, :option07, :option08, :option09, :option10, :text)');
		$stmt->bindValue(':category_id', $schedule['category_id']);
		$stmt->bindValue(':now1',        date('Y-m-d H:i:s'));
		$stmt->bindValue(':now2',        date('Y-m-d H:i:s'));
		$stmt->bindValue(':datetime',    $schedule['datetime']);
		$stmt->bindValue(':enddatetime', $schedule['enddatetime']);
		$stmt->bindValue(':title',       $schedule['title']);
		$stmt->bindValue(':url',         $schedule['url']);
		$stmt->bindValue(':tag',         $schedule['tag']);
		$stmt->bindValue(':place',       $schedule['place']);
		$stmt->bindValue(':option01',    $schedule['option01']);
		$stmt->bindValue(':option02',    $schedule['option02']);
		$stmt->bindValue(':option03',    $schedule['option03']);
		$stmt->bindValue(':option04',    $schedule['option04']);
		$stmt->bindValue(':option05',    $schedule['option05']);
		$stmt->bindValue(':option06',    $schedule['option06']);
		$stmt->bindValue(':option07',    $schedule['option07']);
		$stmt->bindValue(':option08',    $schedule['option08']);
		$stmt->bindValue(':option09',    $schedule['option09']);
		$stmt->bindValue(':option10',    $schedule['option10']);
		$stmt->bindValue(':text',        $schedule['text']);
		$flag = $stmt->execute();
		if (!$flag) {
			freo_error($stmt->errorInfo());
		}
		$schedule['id']           = $freo->pdo->lastInsertId();
	}
	
	//商品ファイル保存
	$file_dir      = FREO_FILE_DIR . 'plugins/schedule_files/' . $schedule['id'] . '/';
	$temporary_dir = FREO_FILE_DIR . 'temporaries/plugins/schedule_files/';

	foreach ($numbers as $number) {
		if (($schedule['file' . $number] and file_exists($temporary_dir . $schedule['file' . $number])) or isset($schedule['file' . $number . '_remove'])) {
			if (file_exists($file_dir)) {
				if ($dir = scandir($file_dir)) {
					foreach ($dir as $data) {
						if (is_file($file_dir . $data) and preg_match("/(\w+)\.\w+$/", $data, $matches)) {
							$filename = $matches[1];

							if ($filename == 'file' . $number) {
								unlink($file_dir . $data);
							}
						}
					}
				} else {
					freo_error('ファイル保存ディレクトリを開けません。');
				}
			}

			if ($schedule['file' . $number]) {
				if (!freo_mkdir($file_dir, FREO_PERMISSION_DIR)) {
					freo_error('ディレクトリ ' . $file_dir . ' を作成できません。');
				}

				if (rename($temporary_dir . $schedule['file' . $number], $file_dir . $schedule['file' . $number])) {
					chmod($file_dir . $schedule['file' . $number], FREO_PERMISSION_FILE);
				} else {
					freo_error('ファイル ' . $temporary_dir . $schedule['file' . $number] . ' を移動できません。');
				}
			}
		}
	}

	//入力データ破棄
	$_SESSION['input'] = array();

	//ログ記録
	if (isset($_GET['id'])) {
		freo_log('予定を編集しました。');
	} else {
		freo_log('予定を新規に登録しました。');
	}

	//予定管理へ移動
	if (isset($_GET['id'])) {
		freo_redirect('schedule/admin?exec=update&id=' . $schedule['id']);
	} else {
		freo_redirect('schedule/admin?exec=insert');
	}

	return;
}

/* 管理画面 | 予定削除 */
function freo_page_schedule_admin_delete()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//パラメータ検証
	if (!isset($_GET['id']) or !preg_match('/^\d+$/', $_GET['id']) or $_GET['id'] < 1) {
		freo_redirect('schedule/admin?error=1');
	}

	//ワンタイムトークン確認
	if (!freo_token('check')) {
		freo_redirect('schedule/admin?error=1');
	}

	//ファイル削除
	freo_rmdir(FREO_FILE_DIR . 'plugins/schedule_files/' . $_GET['id'] . '/');

	//予定削除
	$stmt = $freo->pdo->prepare('DELETE FROM ' . FREO_DATABASE_PREFIX . 'plugin_schedules WHERE id = :id');
	$stmt->bindValue(':id', $_GET['id'], PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		freo_error($stmt->errorInfo());
	}

	//連番リセット
	if (FREO_DATABASE_TYPE == 'mysql') {
		$stmt = $freo->pdo->query('ALTER TABLE ' . FREO_DATABASE_PREFIX . 'plugin_schedules AUTO_INCREMENT = 0');
		if (!$stmt) {
			freo_error($freo->pdo->errorInfo());
		}
	}

	//ログ記録
	freo_log('予定を削除しました。');

	//予定管理へ移動
	freo_redirect('schedule/admin?exec=delete&id=' . $_GET['id']);

	return;
}

/* 管理画面 | カテゴリー管理 */
function freo_page_schedule_admin_category()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//カテゴリー取得
	$stmt = $freo->pdo->query('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_schedule_categories ORDER BY sort, id');
	if (!$stmt) {
		freo_error($freo->pdo->errorInfo());
	}

	$plugin_schedule_categories = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$plugin_schedule_categories[$data['id']] = $data;
	}

	//データ割当
	$freo->smarty->assign(array(
		'token'                    => freo_token('create'),
		'plugin_schedule_categories' => $plugin_schedule_categories
	));

	return;
}

/* 管理画面 | カテゴリー入力 */
function freo_page_schedule_admin_category_form()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//パラメータ検証
	if (!isset($_GET['id']) or !preg_match('/^[\w\-]+$/', $_GET['id'])) {
		$_GET['id'] = null;
	}

	//リクエストメソッドに応じた処理を実行
	if ($_SERVER['REQUEST_METHOD'] == 'POST') {
		//ワンタイムトークン確認
		if (!freo_token('check')) {
			$freo->smarty->append('errors', '不正なアクセスです。');
		}

		//並び順取得
		if ($_POST['plugin_schedule_category']['sort'] != '') {
			$_POST['plugin_schedule_category']['sort'] = mb_convert_kana($_POST['plugin_schedule_category']['sort'], 'n', 'UTF-8');
		}

		//入力データ検証
		if (!$freo->smarty->get_template_vars('errors')) {
			//カテゴリーID
			if ($_POST['plugin_schedule_category']['id'] == '') {
				$freo->smarty->append('errors', 'カテゴリーIDが入力されていません。');
			} elseif (!preg_match('/^[\w\-]+$/', $_POST['plugin_schedule_category']['id'])) {
				$freo->smarty->append('errors', 'カテゴリーIDは半角英数字で入力してください。');
			} elseif (mb_strlen($_POST['plugin_schedule_category']['id'], 'UTF-8') > 80) {
				$freo->smarty->append('errors', 'カテゴリーIDは80文字以内で入力してください。');
			}

			//並び順
			if ($_POST['plugin_schedule_category']['sort'] == '') {
				$freo->smarty->append('errors', '並び順が入力されていません。');
			} elseif (!preg_match('/^[\d]+$/', $_POST['plugin_schedule_category']['sort'])) {
				$freo->smarty->append('errors', '並び順は半角英数字で入力してください。');
			} elseif (mb_strlen($_POST['plugin_schedule_category']['sort'], 'UTF-8') > 10) {
				$freo->smarty->append('errors', '並び順は10文字以内で入力してください。');
			}

			//カテゴリー名
			if ($_POST['plugin_schedule_category']['name'] == '') {
				$freo->smarty->append('errors', 'カテゴリー名が入力されていません。');
			} elseif (mb_strlen($_POST['plugin_schedule_category']['name'], 'UTF-8') > 80) {
				$freo->smarty->append('errors', 'カテゴリー名は80文字以内で入力してください。');
			}

			//説明
			if (mb_strlen($_POST['plugin_schedule_category']['memo'], 'UTF-8') > 5000) {
				$freo->smarty->append('errors', '説明は5000文字以内で入力してください。');
			}
		}

		//エラー確認
		if ($freo->smarty->get_template_vars('errors')) {
			//エラー表示
			$plugin_schedule_category = $_POST['plugin_schedule_category'];
		} else {
			$_SESSION['input'] = $_POST;

			//登録処理へ移動
			freo_redirect('schedule/admin_category_post?freo%5Btoken%5D=' . freo_token('create') . ($_GET['id'] ? '&id=' . $_GET['id'] : ''));
		}
	} else {
		if (!empty($_GET['session']) and !empty($_SESSION['input'])) {
			//入力データ復元
			$plugin_schedule_category = $_SESSION['input']['plugin_schedule_category'];
		} elseif ($_GET['id']) {
			//編集データ取得
			$stmt = $freo->pdo->prepare('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_schedule_categories WHERE id = :id');
			$stmt->bindValue(':id', $_GET['id']);
			$flag = $stmt->execute();
			if (!$flag) {
				freo_error($stmt->errorInfo());
			}

			if ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
				$plugin_schedule_category = $data;
			} else {
				freo_error('指定されたカテゴリーが見つかりません。', '404 Not Found');
			}
		} else {
			//並び順初期値取得
			$stmt = $freo->pdo->query('SELECT MAX(sort) FROM ' . FREO_DATABASE_PREFIX . 'plugin_schedule_categories');
			if (!$stmt) {
				freo_error($freo->pdo->errorInfo());
			}
			$data = $stmt->fetch(PDO::FETCH_NUM);
			$sort = $data[0] + 1;

			//新規データ設定
			$plugin_schedule_category = array(
				'sort' => $sort
			);
		}
	}

	//データ割当
	$freo->smarty->assign(array(
		'token' => freo_token('create'),
		'input' => array(
			'plugin_schedule_category' => $plugin_schedule_category
		)
	));

	return;
}

/* 管理画面 | カテゴリー登録 */
function freo_page_schedule_admin_category_post()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//入力データ確認
	if (empty($_SESSION['input'])) {
		freo_redirect('schedule/admin_category?error=1');
	}

	//ワンタイムトークン確認
	if (!freo_token('check')) {
		freo_redirect('schedule/admin_category?error=1');
	}

	//入力データ取得
	$plugin_schedule_category = $_SESSION['input']['plugin_schedule_category'];

	if ($plugin_schedule_category['memo'] == '') {
		$plugin_schedule_category['memo'] = null;
	}

	//データ登録
	if (isset($_GET['id'])) {
		$stmt = $freo->pdo->prepare('UPDATE ' . FREO_DATABASE_PREFIX . 'plugin_schedule_categories SET modified = :now, sort = :sort, name = :name, memo = :memo WHERE id = :id');
		$stmt->bindValue(':now',  date('Y-m-d H:i:s'));
		$stmt->bindValue(':sort', $plugin_schedule_category['sort'], PDO::PARAM_INT);
		$stmt->bindValue(':name', $plugin_schedule_category['name']);
		$stmt->bindValue(':memo', $plugin_schedule_category['memo']);
		$stmt->bindValue(':id',   $plugin_schedule_category['id']);
		$flag = $stmt->execute();
		if (!$flag) {
			freo_error($stmt->errorInfo());
		}
	} else {
		$stmt = $freo->pdo->prepare('INSERT INTO ' . FREO_DATABASE_PREFIX . 'plugin_schedule_categories VALUES(:id, :now1, :now2, :sort, :name, :memo)');
		$stmt->bindValue(':id',   $plugin_schedule_category['id']);
		$stmt->bindValue(':now1', date('Y-m-d H:i:s'));
		$stmt->bindValue(':now2', date('Y-m-d H:i:s'));
		$stmt->bindValue(':sort', $plugin_schedule_category['sort'], PDO::PARAM_INT);
		$stmt->bindValue(':name', $plugin_schedule_category['name']);
		$stmt->bindValue(':memo', $plugin_schedule_category['memo']);
		$flag = $stmt->execute();
		if (!$flag) {
			freo_error($stmt->errorInfo());
		}
	}

	//入力データ破棄
	$_SESSION['input'] = array();

	//ログ記録
	if (isset($_GET['id'])) {
		freo_log('カテゴリーを編集しました。');
	} else {
		freo_log('カテゴリーを新規に登録しました。');
	}

	//カテゴリー管理へ移動
	if (isset($_GET['id'])) {
		freo_redirect('schedule/admin_category?exec=update&id=' . $plugin_schedule_category['id']);
	} else {
		freo_redirect('schedule/admin_category?exec=insert');
	}

	return;
}

/* 管理画面 | カテゴリー一括編集 */
function freo_page_schedule_admin_category_update()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//ワンタイムトークン確認
	if (!freo_token('check')) {
		freo_redirect('schedule/admin_category?error=1');
	}

	//データ登録
	if (isset($_POST['sort'])) {
		foreach ($_POST['sort'] as $id => $sort) {
			if (!preg_match('/^[\w\-\/]+$/', $id)) {
				continue;
			}
			if (!preg_match('/^\d+$/', $sort)) {
				continue;
			}

			$stmt = $freo->pdo->prepare('UPDATE ' . FREO_DATABASE_PREFIX . 'plugin_schedule_categories SET sort = :sort WHERE id = :id');
			$stmt->bindValue(':sort', $sort, PDO::PARAM_INT);
			$stmt->bindValue(':id',   $id);
			$flag = $stmt->execute();
			if (!$flag) {
				freo_error($stmt->errorInfo());
			}
		}
	}

	//ログ記録
	freo_log('カテゴリーを並び替えました。');

	//カテゴリー管理へ移動
	freo_redirect('schedule/admin_category?exec=sort');

	return;
}

/* 管理画面 | カテゴリー削除 */
function freo_page_schedule_admin_category_delete()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//パラメータ検証
	if (!isset($_GET['id']) or !preg_match('/^[\w\-]+$/', $_GET['id'])) {
		freo_redirect('schedule/admin_category?error=1');
	}

	//ワンタイムトークン確認
	if (!freo_token('check')) {
		freo_redirect('schedule/admin_category?error=1');
	}

	//カテゴリー削除
	$stmt = $freo->pdo->prepare('DELETE FROM ' . FREO_DATABASE_PREFIX . 'plugin_schedule_categories WHERE id = :id');
	$stmt->bindValue(':id', $_GET['id']);
	$flag = $stmt->execute();
	if (!$flag) {
		freo_error($stmt->errorInfo());
	}

	//ログ記録
	freo_log('カテゴリーを削除しました。');

	//カテゴリー管理へ移動
	freo_redirect('schedule/admin_category?exec=delete&id=' . $_GET['id']);

	return;
}

/* 予定個別表示 */
function freo_page_schedule_view()
{
	global $freo;

	//パラメータ検証
	if (!isset($_GET['id']) and isset($freo->parameters[2])) {
		$_GET['id'] = $freo->parameters[2];
	}
	if (!isset($_GET['id']) or !preg_match('/^[\w\-]+$/', $_GET['id'])) {
		freo_error('表示したい予定を指定してください。');
	}

	//予定取得
	$stmt = $freo->pdo->prepare('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_schedules WHERE id = :id');
	$stmt->bindValue(':id', $_GET['id']);
	$flag = $stmt->execute();
	if (!$flag) {
		freo_error($stmt->errorInfo());
	}

	if ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$plugin_schedule = $data;
	} else {
		freo_error('指定された予定が見つかりません。');
	}

	//予定タグ取得
	if ($plugin_schedule['tag']) {
		$plugin_schedule_tags = explode(',', $plugin_schedule['tag']);
	} else {
		$plugin_schedule_tags = array();
	}

	//予定ファイル取得
	$plugin_schedule_files = array();

	$file_dir = FREO_FILE_DIR . 'plugins/schedule_files/' . $_GET['id'] . '/';

	if (file_exists($file_dir)) {
		if ($dir = scandir($file_dir)) {
			foreach ($dir as $data) {
				if (is_file($file_dir . $data) and preg_match("/(\w+)\.\w+$/", $data, $matches)) {
					$plugin_schedule_files[$matches[1]] = $data;
				}
			}
		} else {
			freo_error('商品ファイル保存ディレクトリを開けません。');
		}
	}

	//カテゴリー取得
	$stmt = $freo->pdo->query('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_schedule_categories ORDER BY sort, id');
	if (!$stmt) {
		freo_error($freo->pdo->errorInfo());
	}

	$plugin_schedule_categories = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$plugin_schedule_categories[$data['id']] = $data;
	}

	//データ割当
	$freo->smarty->assign(array(
		'token'                      => freo_token('create'),
		'plugin_schedule'             => $plugin_schedule,
		'plugin_schedule_tags'        => $plugin_schedule_tags,
		'plugin_schedule_files'       => $plugin_schedule_files,
		'plugin_schedule_categories'  => $plugin_schedule_categories,
	));

	if (isset($_GET['type']) and $_GET['type'] == 'embed') {
		//データ出力
		freo_output('plugins/schedule/embed.html');
	}

	return;
}



/* 予定一覧 */
function freo_page_schedule_default()
{
	global $freo;

	//パラメータ検証
	if (!isset($_GET['page']) or !preg_match('/^\d+$/', $_GET['page']) or $_GET['page'] < 1) {
		$_GET['page'] = 1;
	}

	//検索条件設定
	$condition = null;
	if (isset($_GET['word'])) {
		$condition .= ' AND title LIKE ' . $freo->pdo->quote('%' . $_GET['word'] . '%') . ' OR text LIKE ' . $freo->pdo->quote('%' . $_GET['word'] . '%');
	}
	if (isset($_GET['category_id'])) {
		$condition .= ' AND category_id = ' . $freo->pdo->quote($_GET['category_id']);
	}
	if (isset($_GET['tag'])) {
		$condition .= ' AND tag = ' . $freo->pdo->quote($_GET['tag']) . ' OR tag LIKE ' . $freo->pdo->quote($_GET['tag'] . ',%') . ' OR tag LIKE ' . $freo->pdo->quote('%,' . $_GET['tag']) . ' OR tag LIKE ' . $freo->pdo->quote('%,' . $_GET['tag'] . ',%');
	}
	if (isset($_GET['date'])) {
		if (preg_match('/^\d\d\d\d$/', $_GET['date'])) {
			if (FREO_DATABASE_TYPE == 'mysql') {
				$condition .= ' AND DATE_FORMAT(datetime, \'%Y\') = ' . $freo->pdo->quote($_GET['date']);
			} else {
				$condition .= ' AND STRFTIME(\'%Y\', datetime) = ' . $freo->pdo->quote($_GET['date']);
			}
		} elseif (preg_match('/^\d\d\d\d\d\d$/', $_GET['date'])) {
			if (FREO_DATABASE_TYPE == 'mysql') {
				$condition .= ' AND DATE_FORMAT(datetime, \'%Y%m\') = ' . $freo->pdo->quote($_GET['date']);
			} else {
				$condition .= ' AND STRFTIME(\'%Y%m\', datetime) = ' . $freo->pdo->quote($_GET['date']);
			}
		} elseif (preg_match('/^\d\d\d\d\d\d\d\d$/', $_GET['date'])) {
			if (FREO_DATABASE_TYPE == 'mysql') {
				$condition .= ' AND DATE_FORMAT(datetime, \'%Y%m%d\') = ' . $freo->pdo->quote($_GET['date']);
			} else {
				$condition .= ' AND STRFTIME(\'%Y%m%d\', datetime) = ' . $freo->pdo->quote($_GET['date']);
			}
		}
	}


	if ($condition) {
		$condition = ' WHERE id IS NOT NULL ' . $condition;
	}

	//検索条件
	$ascending = false;

		if ($freo->config['plugin']['schedule']['schedule_order'] == 'ascend') {
			$ascending = true;
		}

	if ($ascending == true) {
		$order = ' datetime';
	} else {
		$order = ' datetime DESC';
	}
	
	$titleascending = false;

		if ($freo->config['plugin']['schedule']['schedule_order2'] == 'ascend') {
			$titleascending = true;
		}

	if ($titleascending == true) {
		$titleorder = ' title';
	} else {
		$titleorder = ' title DESC';
	}
	
	//予定取得
	$stmt = $freo->pdo->prepare('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_schedules ' . $condition . ' ORDER BY' . $order . ' , ' . $titleorder . ' LIMIT :start, :limit');
	$stmt->bindValue(':start', intval($freo->config['plugin']['schedule']['default_limit']) * ($_GET['page'] - 1), PDO::PARAM_INT);
	$stmt->bindValue(':limit', intval($freo->config['plugin']['schedule']['default_limit']), PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		freo_error($stmt->errorInfo());
	}

	$plugin_schedules = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$plugin_schedules[$data['id']] = $data;
	}

	//予定ID取得
	$plugin_schedule_keys = array_keys($plugin_schedules);

	//予定タグ取得
	$plugin_schedule_tags = array();
	foreach ($plugin_schedule_keys as $plugin_schedule) {
		if (!$plugin_schedules[$plugin_schedule]['tag']) {
			continue;
		}

		$plugin_schedule_tags[$plugin_schedule] = explode(',', $plugin_schedules[$plugin_schedule]['tag']);
	}

	//ファイル取得
	$plugin_schedule_files = array();
	foreach ($plugin_schedule_keys as $plugin_schedule) {
		$file_dir = FREO_FILE_DIR . 'plugins/schedule_files/' . $plugin_schedule . '/';

		if (file_exists($file_dir)) {
			if ($dir = scandir($file_dir)) {
				foreach ($dir as $data) {
					if (is_file($file_dir . $data) and preg_match("/(\w+)\.\w+$/", $data, $matches)) {
						$plugin_schedule_files[$plugin_schedule][$matches[1]] = $data;
					}
				}
			} else {
				freo_error('商品ファイル保存ディレクトリを開けません。');
			}
		}
	}

	//予定数・ページ数取得
	$stmt = $freo->pdo->query('SELECT COUNT(*) FROM ' . FREO_DATABASE_PREFIX . 'plugin_schedules ' . $condition);
	if (!$stmt) {
		freo_error($freo->pdo->errorInfo());
	}

	$data                  = $stmt->fetch(PDO::FETCH_NUM);
	$plugin_schedule_count = $data[0];
	$plugin_schedule_page  = ceil($plugin_schedule_count / $freo->config['plugin']['schedule']['default_limit']);

	//カテゴリー取得
	$stmt = $freo->pdo->query('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_schedule_categories ORDER BY sort, id');
	if (!$stmt) {
		freo_error($freo->pdo->errorInfo());
	}

	$plugin_schedule_categories = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$plugin_schedule_categories[$data['id']] = $data;
	}

	//データ割当
	$freo->smarty->assign(array(
		'token'                 => freo_token('create'),
		'plugin_schedules'      => $plugin_schedules,
		'plugin_schedule_tags'  => $plugin_schedule_tags,
		'plugin_schedule_files' => $plugin_schedule_files,
		'plugin_schedule_count' => $plugin_schedule_count,
		'plugin_schedule_page'  => $plugin_schedule_page,
		'plugin_schedule_categories'  => $plugin_schedule_categories
	));

	return;
}

?>
